@@ -16,14 +16,39 @@
 Transforming Python basic types to Python dbus types.
 """
 
+import functools
+
 import dbus
 
 from dbus_signature_pyparsing import Parser
 
 from ._errors import IntoDPValueError
 
+def _wrapper(func):
+    """
+    Wraps a generated function so that it catches all Type- and ValueErrors
+    and raises IntoDPValueErrors.
 
-class ToDbusXformer(Parser):
+    :param func: the transforming function
+    """
+
+    @functools.wraps(func)
+    def the_func(expr):
+        """
+        The actual function.
+
+        :param object expr: the expression to be xformed to dbus-python types
+        """
+        try:
+            return func(expr)
+        except (TypeError, ValueError) as err:
+            raise IntoDPValueError(expr, "expr", "could not be transformed") \
+               from err
+
+    return the_func
+
+
+class _ToDbusXformer(Parser):
     """
     Class which extends a Parser to yield a function that yields
     a function that transforms a value in base Python types to a correct value
@@ -108,7 +133,7 @@ class ToDbusXformer(Parser):
                    0 if elements == [] \
                    else max(max(x, y) for ((_, x), (_, y)) in elements)
                 (obj_level, func_level) = \
-                   ToDbusXformer._variant_levels(level, variant)
+                   _ToDbusXformer._variant_levels(level, variant)
                 return (
                    dbus.types.Dictionary(
                       ((x, y) for ((x, _), (y, _)) in elements),
@@ -134,10 +159,16 @@ class ToDbusXformer(Parser):
                 :returns: a dbus Array of transformed values and variant level
                 :rtype: Array * int
                 """
+                if isinstance(a_list, dict):
+                    raise IntoDPValueError(
+                       a_list,
+                       "a_list",
+                       "is a dict, must be an array"
+                    )
                 elements = [func(x) for x in a_list]
                 level = 0 if elements == [] else max(x for (_, x) in elements)
                 (obj_level, func_level) = \
-                   ToDbusXformer._variant_levels(level, variant)
+                   _ToDbusXformer._variant_levels(level, variant)
 
                 return (
                    dbus.types.Array(
@@ -177,6 +208,12 @@ class ToDbusXformer(Parser):
             :rtype: Struct * int
             :raises IntoDPValueError:
             """
+            if isinstance(a_list, dict):
+                raise IntoDPValueError(
+                   a_list,
+                   "a_list",
+                   "must be a simple sequence, is a dict"
+                )
             if len(a_list) != len(funcs):
                 raise IntoDPValueError(
                    a_list,
@@ -187,7 +224,7 @@ class ToDbusXformer(Parser):
             elements = [f(x) for (f, x) in zip(funcs, a_list)]
             level = 0 if elements == [] else max(x for (_, x) in elements)
             (obj_level, func_level) = \
-                ToDbusXformer._variant_levels(level, variant)
+                _ToDbusXformer._variant_levels(level, variant)
             return (
                dbus.types.Struct(
                   (x for (x, _) in elements),
@@ -215,62 +252,82 @@ class ToDbusXformer(Parser):
             :returns: a tuple of a dbus object and the variant level
             :rtype: dbus object * int
             """
-            (obj_level, func_level) = ToDbusXformer._variant_levels(0, variant)
+            (obj_level, func_level) = _ToDbusXformer._variant_levels(0, variant)
             return (klass(v, variant_level=obj_level), func_level)
 
         return lambda: (the_func, symbol)
 
     def __init__(self):
-        super(ToDbusXformer, self).__init__()
+        super(_ToDbusXformer, self).__init__()
 
         self.BYTE.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Byte, 'y')
+           _ToDbusXformer._handleBaseCase(dbus.types.Byte, 'y')
         )
         self.BOOLEAN.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Boolean, 'b')
+           _ToDbusXformer._handleBaseCase(dbus.types.Boolean, 'b')
         )
         self.INT16.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Int16, 'n')
+           _ToDbusXformer._handleBaseCase(dbus.types.Int16, 'n')
         )
         self.UINT16.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.UInt16, 'q')
+           _ToDbusXformer._handleBaseCase(dbus.types.UInt16, 'q')
         )
         self.INT32.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Int32, 'i')
+           _ToDbusXformer._handleBaseCase(dbus.types.Int32, 'i')
         )
         self.UINT32.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.UInt32, 'u')
+           _ToDbusXformer._handleBaseCase(dbus.types.UInt32, 'u')
         )
         self.INT64.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Int64, 'x')
+           _ToDbusXformer._handleBaseCase(dbus.types.Int64, 'x')
         )
         self.UINT64.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.UInt64, 't')
+           _ToDbusXformer._handleBaseCase(dbus.types.UInt64, 't')
         )
         self.DOUBLE.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Double, 'd')
+           _ToDbusXformer._handleBaseCase(dbus.types.Double, 'd')
         )
         self.UNIX_FD.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.UnixFd, 'h')
+           _ToDbusXformer._handleBaseCase(dbus.types.UnixFd, 'h')
         )
         self.STRING.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.String, 's')
+           _ToDbusXformer._handleBaseCase(dbus.types.String, 's')
         )
         self.OBJECT_PATH.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.ObjectPath, 'o')
+           _ToDbusXformer._handleBaseCase(dbus.types.ObjectPath, 'o')
         )
         self.SIGNATURE.setParseAction(
-           ToDbusXformer._handleBaseCase(dbus.types.Signature, 'g')
+           _ToDbusXformer._handleBaseCase(dbus.types.Signature, 'g')
         )
 
         self.VARIANT.setParseAction(self._handleVariant)
 
-        self.ARRAY.setParseAction(ToDbusXformer._handleArray)
+        self.ARRAY.setParseAction(_ToDbusXformer._handleArray)
 
-        self.STRUCT.setParseAction(ToDbusXformer._handleStruct)
+        self.STRUCT.setParseAction(_ToDbusXformer._handleStruct)
 
 
-_XFORMER = ToDbusXformer()
+_XFORMER = _ToDbusXformer()
+
+
+def xformers(sig):
+    """
+    Get the list of xformer functions for the given signature.
+
+    :param str sig: a signature
+    :returns: a list of xformer functions for the given signature.
+    :rtype: list of tuple of a function * str
+
+    Each function catches all TypeErrors it encounters and raises
+    corresponding IntoDPValueError exceptions.
+    """
+    return list(
+       map(
+          lambda x: (_wrapper(x[0]), x[1]),
+          _XFORMER.PARSER.parseString(sig, parseAll=True)
+       )
+    )
+
 
 def xformer(signature):
     """
@@ -281,8 +338,7 @@ def xformer(signature):
     :rtype: (list of object) -> (list of object)
     """
 
-    funcs = \
-       [f for (f, _) in _XFORMER.PARSER.parseString(signature, parseAll=True)]
+    funcs = [f for (f, _) in xformers(signature)]
 
     def the_func(objects):
         """
@@ -294,6 +350,13 @@ def xformer(signature):
         :returns: transformed objects
         :rtype: list of object (in dbus types)
         """
+        if len(objects) != len(funcs):
+            raise IntoDPValueError(
+               objects,
+               "objects",
+               "must have exactly %u items, has %u" % \
+                  (len(funcs), len(objects))
+            )
         return [x for (x, _) in (f(a) for (f, a) in zip(funcs, objects))]
 
     return the_func
